package com.example.assignment1;

import android.os.Bundle;
import android.text.InputFilter;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

public class SettingsActivity extends AppCompatActivity {

    protected Button saveButton;
    protected Toolbar myToolbar;

    protected int maxLength = 16;

    protected CounterButtonSharedPreferenceHelper firstCounterButtonSharedPrefHelper;
    protected CounterButtonSharedPreferenceHelper secondCounterButtonSharedPrefHelper;
    protected CounterButtonSharedPreferenceHelper thirdCounterButtonSharedPrefHelper;

    protected MaxCountSharedPreferenceHelper maxCountSharedPrefHelper;

    protected EditText counter_1_name_edit_text;
    protected EditText counter_2_name_edit_text;
    protected EditText counter_3_name_edit_text;

    protected EditText maximum_counts_edit_text;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);

        myToolbar = (Toolbar) findViewById(R.id.settings_toolbar);
        saveButton = (Button) findViewById(R.id.saveButton);

        counter_1_name_edit_text = (EditText) findViewById(R.id.counter_1_edit_text);
        counter_2_name_edit_text = (EditText) findViewById(R.id.counter_2_edit_text);
        counter_3_name_edit_text = (EditText) findViewById(R.id.counter_3_edit_text);
        maximum_counts_edit_text = (EditText) findViewById(R.id.maximum_counts_edit_text);

        counter_1_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
        counter_2_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});
        counter_3_name_edit_text.setFilters(new InputFilter[]{new InputFilter.LengthFilter(maxLength)});

        firstCounterButtonSharedPrefHelper = new CounterButtonSharedPreferenceHelper(SettingsActivity.this, CounterButton.FIRST);
        secondCounterButtonSharedPrefHelper = new CounterButtonSharedPreferenceHelper(SettingsActivity.this, CounterButton.SECOND);
        thirdCounterButtonSharedPrefHelper = new CounterButtonSharedPreferenceHelper(SettingsActivity.this, CounterButton.THIRD);
        maxCountSharedPrefHelper = new MaxCountSharedPreferenceHelper(SettingsActivity.this);

        // enable toolbar/actionbar
        setSupportActionBar(myToolbar);

        assert getSupportActionBar() != null;
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);   //show back button

        goToSettingsActivityDisplayMode();
    }

    @Override
    protected void onStart()
    {
        super.onStart();

        if(
                firstCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST) == null ||
                        secondCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND) == null ||
                        thirdCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD) == null ||
                        maxCountSharedPrefHelper.getMaxCount() == null) {

            goToSettingsActivityEditMode();

        } else {
            counter_1_name_edit_text.setHint(firstCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST));
            counter_2_name_edit_text.setHint(secondCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND));
            counter_3_name_edit_text.setHint(thirdCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD));
            maximum_counts_edit_text.setHint(maxCountSharedPrefHelper.getMaxCount());
        }

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.edit_settings) {
            // Handle the settings item click (e.g., open a settings activity).
            goToSettingsActivityEditMode();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.settings_activity_menu, menu);
        return true;
    }

    protected void goToSettingsActivityDisplayMode(){
        // no save button in Display Mode
        saveButton.setVisibility(View.INVISIBLE);

        // in DISPLAY mode, it's not possible to edit counter button names
        counter_1_name_edit_text.setEnabled(false);
        counter_2_name_edit_text.setEnabled(false);
        counter_3_name_edit_text.setEnabled(false);
        maximum_counts_edit_text.setEnabled(false);
    }

    protected void goToSettingsActivityEditMode(){

        saveButton.setVisibility(View.VISIBLE);

        counter_1_name_edit_text.setEnabled(true);
        counter_2_name_edit_text.setEnabled(true);
        counter_3_name_edit_text.setEnabled(true);
        maximum_counts_edit_text.setEnabled(true);

        counter_1_name_edit_text.setText(firstCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.FIRST));
        counter_2_name_edit_text.setText(secondCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.SECOND));
        counter_3_name_edit_text.setText(thirdCounterButtonSharedPrefHelper.getCounterButtonName(CounterButton.THIRD));
        maximum_counts_edit_text.setText(maxCountSharedPrefHelper.getMaxCount());

        saveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // counter button strings are checked statically in the activity XML file
                if (check_max_count_input()){
                    // Get the text from EditText fields
                    String text1 = counter_1_name_edit_text.getText().toString();
                    String text2 = counter_2_name_edit_text.getText().toString();
                    String text3 = counter_3_name_edit_text.getText().toString();
                    String maximum_count = maximum_counts_edit_text.getText().toString();

                    firstCounterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.FIRST, text1);
                    secondCounterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.SECOND, text2);
                    thirdCounterButtonSharedPrefHelper.saveCounterButtonName(CounterButton.THIRD, text3);
                    maxCountSharedPrefHelper.saveMaxCount(maximum_count);

                    goToSettingsActivityDisplayMode();
                }
            }
        });
    }

    protected Boolean check_max_count_input(){
        try {
            int countInput = Integer.parseInt(maximum_counts_edit_text.getText().toString());
                if (countInput < 5 || countInput > 200){
                    Toast.makeText(getApplicationContext(), "Number must be between 5 and 200 inclusive", Toast.LENGTH_SHORT).show();
                    return false;
                }
        } catch (NumberFormatException e){
            Toast.makeText(getApplicationContext(), "Number must be between 5 and 200 inclusive", Toast.LENGTH_SHORT).show();
            return false;
        }

        return true;
    }
}